@@ -116,3 +116,57 @@ class Backend(object):
     @abstractmethod
     def list_model_releases(self, model_id, version):
         pass
+
+    @abstractmethod
+    def get_mlflow_experiment(self, experiment_id):
+        pass
+
+    @abstractmethod
+    def get_mlproject_file(self):
+        pass
+
+    @abstractmethod
+    def get_experiment(self, experiment):
+        pass
+
+    @abstractmethod
+    def create_experiment(self, name, model_id):
+        pass
+
+    @abstractmethod
+    def create_mlflow_experiment(self, name, artifact_location):
+        pass
+
+    @abstractmethod
+    def create_mlflow_run(self, name, artifact_uri, run_uuid, experiment_id, source_type,
+                          source_name, entry_point_name, user_id, status, start_time, end_time,
+                          source_version, lifecycle_stage):
+        pass
+
+    @abstractmethod
+    def get_experiment_by_mlflow_experiment(self, experiment_id):
+        pass
+
+    @abstractmethod
+    def get_experiment_by_mlflow_run(self, run_id):
+        pass
+
+    @abstractmethod
+    def get_mlflow_run(self, run_id):
+        pass
+
+    @abstractmethod
+    def log_param(self, run_id, params):
+        pass
+
+    @abstractmethod
+    def log_metric(self, run_id, metric, value, is_nan):
+        pass
+
+    @abstractmethod
+    def set_tag(self, run_id, tag):
+        pass
+
+    @abstractmethod
+    def update_run_info(self, run_id, run_status, end_time):
+        pass
