@@ -30,6 +30,7 @@ from waveletai.constants import DataType, ModelRegisterMode
 from waveletai.utils.storage_utils import UploadEntry, scan_upload_entries, SilentProgressIndicator, \
     LoggingProgressIndicator, upload_to_storage, dataset_upload_to_storage
 from waveletai.utils.datastream import FileStream, FileChunkStream, FileChunk
+from waveletai.experiment import Experiment, MLFlowExperiment, MLFlowRun
 
 _logger = logging.getLogger(__name__)
 
@@ -296,7 +297,8 @@ class HostedBackend(Backend):
             name = os.getenv(envs.USER_NAME)
         if pwd is None:
             pwd = os.getenv(envs.USER_PWD)
-        if all([name, pwd]):
+        api_token = os.getenv(envs.API_TOKEN)
+        if all([name, pwd]) or api_token:
             pass
         else:
             raise MissingNameOrPwdException()
@@ -308,15 +310,19 @@ class HostedBackend(Backend):
             system=platform.platform(),
             python_version=platform.python_version())
         headers = {'User-Agent': user_agent}
-        res = self._session.post(
-            url=f'{self.api_address}/account/users/login/',
-            json={'username': name, 'password': pwd},
-            headers=headers,
-        )
-        if res.json()["message"]:
-            raise LoginFailed(res.json()["message"])
-        print("WaveletAI Backend connected")
-        self._session.headers.update({"X-TOKEN": res.json()["data"]["token"]})
+        if api_token is None:
+            res = self._session.post(
+                url=f'{self.api_address}/account/users/login/',
+                json={'username': name, 'password': pwd},
+                headers=headers,
+            )
+            if res.json()["message"]:
+                raise LoginFailed(res.json()["message"])
+            print("WaveletAI Backend connected")
+            api_token = res.json()["data"]["token"]
+            os.environ.update({envs.API_TOKEN: api_token})
+
+        self._session.headers.update({"X-TOKEN": api_token})
 
     @with_api_exceptions_handler
     def _create_authenticator(self, api_token, ssl_verify):
@@ -356,7 +362,7 @@ class HostedBackend(Backend):
         print(f"Dataset id = {res.json()['data']['id']} created succ")
 
         dict = res.json()['data']
-        self.upload_dataset(dict['id'], path)
+        self.upload_dataset_artifacts(dict['id'], path)
         return Dataset(self, dict['id'], dict['name'], dict['desc'], dict['zone'], dict['dimension'], dict['json_data'],
                        dict['create_time'], dict['type'], dict['create_user_id'], dict['update_time'],
                        dict['update_user_id'])
@@ -553,14 +559,19 @@ class HostedBackend(Backend):
                     if '.' not in path:
                         resultinfo, dir = self._list_dir_artifacts(dataset_id, path, [], [])
                         for i in dir:
+                            print(i)
                             if not os.path.exists(destination + i):
                                 os.makedirs(destination + i)
                         for asset in artifacts:
+                            print(asset.name)
                             for res in resultinfo:
+                                print(res)
                                 if asset.name in res:
                                     if '/' in res:
                                         res_des = res.rsplit('/', 1)
                                         res_des = res_des[0]
+                                    else:
+                                        res_des = ''
                                     urls.append((asset.path, destination + res_des, asset.name))
                     else:
                         for asset in artifacts:
@@ -592,9 +603,11 @@ class HostedBackend(Backend):
         :param destination:
         :return:
         """
+
         if not destination:
             destination = os.getcwd()
-
+        if not destination.endswith("/"):
+            destination = destination + "/"
         if not os.path.exists(destination):
             os.makedirs(destination)
         elif not os.path.isdir(destination):
@@ -614,8 +627,11 @@ class HostedBackend(Backend):
                     for asset in artifacts:
                         for res in resultinfo:
                             if res in asset.path:
-                                res_des = res.rsplit('/', 1)
-                                res_des = res_des[0]
+                                if '/' in res:
+                                    res_des = res.rsplit('/', 1)
+                                    res_des = res_des[0]
+                                else:
+                                    res_des = ''
                                 urls.append((asset.path, destination + res_des, asset.name))
                     pool.map(self._url_download, list(set(urls)))
                     pool.close()
@@ -719,7 +735,7 @@ class HostedBackend(Backend):
         :param model_id:
         :return: Model instance
         """
-        res = self._session.get(url=f'{self.api_address}/data/model/{model_id}/')
+        res = self._session.get(url=f'{self.api_address}/training/model/{model_id}/')
         if res.json()["message"]:
             raise Exception(res.json()["message"])
         dict = res.json()['data']
@@ -992,3 +1008,324 @@ class HostedBackend(Backend):
         data = res.json()['data']
         return ModelVersion(self, data['id'], data['version'], data['desc'], data['model_id'], data['model_name'],
                             data['create_user_name'], data['create_time'], data['create_user_id'], data['mode'])
+
+    @with_api_exceptions_handler
+    def get_experiment(self, experiment_id):
+        """
+        获取实验对象
+        :param experiment_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/experiment/{experiment_id}/')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return Experiment(self, id=data['id'], name=data['name'], desc=data['desc'], param_json=data['param_json'],
+                          host_id=data['host_id'], model_id=data['model_id'], dataset_id=data['dataset_id'],
+                          start_time=data['start_time'], end_time=data['end_time'], state=data['state'],
+                          best_run_id=data['best_run_id'], create_time=data['create_time'],
+                          create_user_id=data['create_user_id'], mlflow_experiments_id=data['mlflow_experiments_id'],
+                          ml_experiment=data['ml_experiment'])
+
+    @with_api_exceptions_handler
+    def get_mlflow_experiment(self, experiment_id):
+        """
+        获取mlflow实验对象
+        :param experiment_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/mlflow/experiment/{experiment_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return MLFlowExperiment(self, data['experiment_id'], data['name'], data['artifact_location'],
+                                data['lifecycle_stage'])
+
+    @with_api_exceptions_handler
+    def get_mlproject_file(self, model_id):
+        """
+        根据模型id获取mlproject的内容
+        :param model_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/model/{model_id}/file')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return data
+
+    @with_api_exceptions_handler
+    def create_experiment(self, name, model_id, params=None, host_id=None, dataset_id=None, docker_args=None,
+                          feature_id=None, is_train=None, desc=None, mlexp_id=None, artifact_location=None,
+                          online_train=False):
+        """
+        创建实验
+        :param name:
+        :param model_id:
+        :param params:
+        :param host_id:
+        :param dataset_id:
+        :param docker_args:
+        :param feature_id:
+        :param is_train:
+        :param desc:
+        :param mlexp_id:
+        :param online_train:
+        :param artifact_location:
+        :return:
+        """
+
+        def _get_type(ins):
+            if isinstance(ins, int):
+                return 'int'
+            if isinstance(ins, float):
+                return 'float'
+            if isinstance(ins, str):
+                return 'str'
+            if isinstance(ins, list):
+                return 'list'
+            if isinstance(ins, dict):
+                return 'dict'
+            return None
+
+        param_json = []
+        if params is not None:
+            for k, v in params.items():
+                _d = {'label': k, 'prop': k, 'default': v}
+                if v:
+                    _d.update({'type': _get_type(v)})
+                param_json.append(_d)
+
+        data = dict(name=name, model_id=model_id,
+                    param_json=param_json,
+                    host_id=host_id,
+                    dataset_id=dataset_id,
+                    docker_args=docker_args,
+                    feature_id=feature_id,
+                    is_train=is_train,
+                    desc=desc,
+                    # mlexp_id=mlexp_id,
+                    artifact_location=artifact_location,
+                    online_train=False
+                    )
+        res = self._session.post(url=f'{self.api_address}/training/experiment/', json=data)
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return Experiment(self, id=data['id'], name=data['name'], desc=data['desc'], host_id=data['host_id'],
+                          param_json=data['param_json'], model_id=data['model_id'], dataset_id=data['dataset_id'],
+                          start_time=data['start_time'], end_time=data['end_time'], state=data['state'],
+                          best_run_id=data['best_run_id'], create_time=data['create_time'],
+                          create_user_id=data['create_user_id'], mlflow_experiments_id=data['mlflow_experiments_id'],
+                          ml_experiment=data['ml_experiment'])
+
+    @with_api_exceptions_handler
+    def create_mlflow_experiment(self, name, artifact_location):
+        """
+        创建mlflow实验
+        :param name:
+        :param artifact_location:
+        :return:
+        """
+        data = dict(name=name, artifact_location=artifact_location)
+        res = self._session.post(url=f'{self.api_address}/training/run/ml-experiment/', json=data)
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return data['mlflow_experiment_id']
+
+    @with_api_exceptions_handler
+    def get_experiment_by_mlflow_experiment(self, experiment_id):
+        """
+        根据mlflow 查询wai experiment
+        :param experiment_id:
+        :return:
+        """
+        res = self._session.get(f'{self.api_address}/training/experiment/mlflow/{experiment_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return Experiment(self, id=data['id'], name=data['name'], desc=data['desc'], host_id=data['host_id'],
+                          param_json=data['param_json'],
+                          model_id=data['model_id'], dataset_id=data['dataset_id'], start_time=data['start_time'],
+                          end_time=data['end_time'], state=data['state'], best_run_id=data['best_run_id'],
+                          create_time=data['create_time'], create_user_id=data['create_user_id'],
+                          mlflow_experiments_id=data['mlflow_experiments_id'], ml_experiment=data['ml_experiment'])
+
+    @with_api_exceptions_handler
+    def get_experiment_by_mlflow_run(self, run_id):
+        """
+        根据mlflow run_id 查询 wai experiment
+        :param run_id:
+        :return:
+        """
+        res = self._session.get(f'{self.api_address}/training/experiment/mlflow/run/{run_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return Experiment(self, id=data['id'], name=data['name'], desc=data['desc'], host_id=data['host_id'],
+                          param_json=data['param_json'],
+                          model_id=data['model_id'], dataset_id=data['dataset_id'], start_time=data['start_time'],
+                          end_time=data['end_time'], state=data['state'], best_run_id=data['best_run_id'],
+                          create_time=data['create_time'], create_user_id=data['create_user_id'],
+                          mlflow_experiments_id=data['mlflow_experiments_id'], ml_experiment=data['ml_experiment'])
+
+    @with_api_exceptions_handler
+    def create_mlflow_run(self, name, artifact_uri, run_uuid, experiment_id, source_type,
+                          source_name, entry_point_name, user_id, status, start_time, end_time,
+                          source_version, lifecycle_stage):
+        """
+        创建mlflow run
+        :param name:
+        :param artifact_uri:
+        :param run_uuid:
+        :param experiment_id:
+        :param source_type:
+        :param source_name:
+        :param entry_point_name:
+        :param user_id:
+        :param status:
+        :param start_time:
+        :param end_time:
+        :param source_version:
+        :param lifecycle_stage:
+        :return:
+        """
+        data = dict(name=name, artifact_uri=artifact_uri, run_uuid=run_uuid, experiment_id=experiment_id,
+                    source_type=source_type, source_name=source_name, entry_point_name=entry_point_name,
+                    user_id=user_id, status=status, start_time=start_time, end_time=end_time,
+                    source_version=source_version, lifecycle_stage=lifecycle_stage)
+        res = self._session.post(url=f'{self.api_address}/training/run/', json=data)
+
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return MLFlowRun(self, **data)
+
+    @with_api_exceptions_handler
+    def get_mlflow_run(self, run_id):
+        """
+        查询mlflow run
+        :param run_id:
+        :return:
+        """
+        res = self._session.get(f'{self.api_address}/training/mlflow/run/{run_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        info = data['run_info']
+        metrics, params, tags = data['run_data']['metrics'], data['run_data']['params'], data['run_data']['tags']
+
+        return MLFlowRun(self, name='',
+                         artifact_uri=info['artifact_uri'],
+                         run_uuid=info['run_uuid'],
+                         experiment_id=info['experiment_id'],
+                         source_type=tags.get('mlflow.source.type'),
+                         source_name=tags.get('mlflow.source.name'),
+                         entry_point_name=tags.get('mlflow.project.entryPoint'),
+                         user_id=tags['mlflow.user'],
+                         status=info['status'],
+                         start_time=info['start_time'],
+                         end_time=info['end_time'],
+                         source_version='',
+                         lifecycle_stage=info['lifecycle_stage']
+                         )
+
+    @with_api_exceptions_handler
+    def log_param(self, run_id, param):
+        """
+        log param
+        :param run_id:
+        :param param:
+        :return:
+        """
+        data = {
+            'key': param.key,
+            'value': param.value
+        }
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/log_param', json=data)
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+
+    @with_api_exceptions_handler
+    def log_metric(self, run_id, metric, value, is_nan):
+        """
+        log metric
+        :param run_id:
+        :param metric:
+        :return:
+        """
+        data = {
+            'key': metric.key,
+            'value': value,
+            'timestamp': metric.timestamp,
+            'step': metric.step,
+            'is_nan': is_nan
+        }
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/log_metric', json=data)
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+
+    @with_api_exceptions_handler
+    def set_tag(self, run_id, tag):
+        """
+        log tag
+        :param run_id:
+        :param tag:
+        :return:
+        """
+        data = {
+            'key': tag.key,
+            'value': tag.value
+        }
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/set_tag', json=data)
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+
+    # @with_api_exceptions_handler
+    # def log_batch(self, run_id, metrics, params, tags):
+    #     """
+    #     写入mlflow log
+    #     :param run_id:
+    #     :param metrics:
+    #     :param params:
+    #     :param tags:
+    #     :return:
+    #     """
+    #     data = {
+    #         'metrics': metrics,
+    #         'params': params,
+    #         'tags': tags
+    #     }
+    #     res = self._session.post(url=f'{self.api_address}/training/mlflow/run/{run_id}/log_batch')
+    #     if res.json()['message']:
+    #         raise Exception(res.json()['message'])
+    #     data = res.json()['data']
+    #     return
+
+    @with_api_exceptions_handler
+    def update_run_info(self, run_id, run_status, end_time):
+        """
+        更新run
+        :param run_id:
+        :param run_status:
+        :param end_time:
+        :return:
+        """
+        data = {
+            'run_status': run_status,
+            'end_time': end_time
+        }
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}', json=data)
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return MLFlowRun(self, **data)
