@@ -171,3 +171,8 @@ class UnsupportedClientVersion(WaveletAIException):
                 version,
                 "==" + str(maxVersion) if maxVersion else ">=" + str(minVersion)
             ))
+
+
+class InvalidMLProjectFile(WaveletAIException):
+    def __init__(self):
+        super(InvalidMLProjectFile, self).__init__('the content of MLPorject is invalid')
