@@ -10,7 +10,14 @@
 """
 
 import logging
+import yaml
+import os
+
 from waveletai.constants import ModelRegisterMode
+from waveletai.exceptions import InvalidMLProjectFile
+from waveletai.utils import gen_id
+import mlflow
+from waveletai.experiment import Experiment
 
 _logger = logging.getLogger(__name__)
 
@@ -109,6 +116,40 @@ class Model(object):
         """search for a specific stage or desc and list its version details"""
         pass
 
+    def mlflow_project_file(self, mlproject_path=None):
+        """
+        获取模型的MLProject
+        :param mlproject_path: 如果传入了路径, 将读取该路径指向的mlproject描述
+        :return:
+        """
+        if mlproject_path:
+            os.path.exists(mlproject_path)
+            with open(mlproject_path, 'rb') as f:
+                return yaml.load(f, Loader=yaml.FullLoader)
+
+        return self._backend.get_mlproject_file(self.id)
+
+    def create_experiment(self, name, params=None):
+        """
+        创建一个实验, 先创建mlflow的experiment, 再创建wai平台的experiment
+        :param name:
+        :param params: 训练参数字典 {key:value, key2:value2, ...}
+        :return:
+        """
+        return self._backend.create_experiment(name=name, model_id=self.id,
+                                               params=params,
+                                               docker_args={},
+                                               is_train=True
+                                               )
+
+    def get_experiment(self, experiment_id):
+        """
+        获取实验对象
+        :param experiment_id:
+        :return:
+        """
+        return self._backend.get_experiment(experiment_id)
+
 
 class ModelVersion(object):
     def __init__(self, backend, id, version, desc, model_id, model_name, mode, create_user_name, create_time,
